<?php

namespace Drupal\javascript\Plugin\Field\FieldWidget;

use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Field\FieldItemListInterface;

/**
 * Plugin implementation of the 'javascript_widget' widget.
 *
 * @FieldWidget(
 *   id = "javascript_widget",
 *   label = @Translation("Libraries, HTML, and Javascript"),
 *   field_types = {
 *     "javascript"
 *   }
 * )
 */
class JavascriptWidget extends WidgetBase {

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    return [
      '#attached' => [
        'library' => [
          'javascript/babel',
          'javascript/codemirror',
        ],
      ],
      'libraries' => [
        '#type' => 'textfield',
        '#title' => $this->t('Libraries Attached'),
        '#size' => 60,
        '#description' => 'A comma separated list of drupal libraries. Example: javascript/react, javascript/highlight',
        '#default_value' => isset($items[$delta]->libraries) ? $items[$delta]->libraries : NULL,
      ],
      'html' => [
        '#type' => 'textarea',
        '#title' => $this->t('Application HTML'),
        '#rows' => 25,
        '#resizable' => 'both',
        '#attributes' => [
          'class' => ['codemirror-html'],
        ],
        '#default_value' => isset($items[$delta]->html) ? $items[$delta]->html : NULL,
      ],
      'script' => [
        '#type' => 'textarea',
        '#title' => $this->t('Javascript Code'),
        '#description' => 'Accepts ES6 Javascript, Typescript, JSX, and TSX. Transpiled to ES5 Javascript.',
        '#rows' => 25,
        '#resizable' => 'both',
        '#attributes' => [
          'class' => [
            'codemirror-javascript',
            'babel-compile-source',
          ],
        ],
        '#default_value' => isset($items[$delta]->script) ? $items[$delta]->script : NULL,
      ],
      'revision' => [
        '#type' => 'hidden',
        '#default_value' => isset($items[$delta]->revision) ? $items[$delta]->revision : 0,
      ],
      'compiled' => [
        '#type' => 'hidden',
        '#attributes' => [
          'class' => ['babel-compile-target'],
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {
    $parent = $form_state->getFormObject()->getEntity();
    $parent_id = $parent->id();
    $target_dir = 'public://js/transpiled/';
    \Drupal::service('file_system')->prepareDirectory($target_dir, FileSystemInterface::CREATE_DIRECTORY);
    $js_name = $target_dir . $this->fieldDefinition->getTargetBundle() . '_' . $parent_id . '_' . $this->fieldDefinition->getName() . '_';
    foreach ($values as $key => $value) {
      $values[$key]['revision']++;
      $this_js_name = $js_name . $key . 'r' . $values[$key]['revision'] . '.js';
      file_put_contents($this_js_name, $value["compiled"]);
    }
    return $values;
  }

}
